@@ -3,7 +3,7 @@ main application classes and widgets for GUIApp-conform Kivy apps
 =================================================================
 
 this ae portion is providing additional :ref:`config-variables` and some useful constants, various enhanced widget
-classes, two application classes (:class:`FrameworkApp` and :class:`KivyMainApp`) and a i18n wrapper (:func:`get_txt`),
+classes, two application classes (:class:`FrameworkApp` and :class:`KivyMainApp`) and an i18n wrapper (:func:`get_txt`),
 adding translatable f-strings to the python and kv code of your app.
 
 
@@ -57,7 +57,7 @@ a Python application based on the `kivy framework <https://kivy.org>`_.
 :class:`KivyMainApp` is based on the following classes:
 
 * the abstract base class :class:`~ae.gui_app.MainAppBase` which adds :ref:`application status`,
-  :ref:`app-state-variables` and :ref:`app-state-constants`), :ref:`application flow` and :ref:`application events`.
+  :ref:`app-state-variables`, :ref:`app-state-constants`, :ref:`application flow` and :ref:`application events`.
 * :class:`~ae.console.ConsoleApp` is adding :ref:`config-files`, :ref:`config-variables` and :ref:`config-options`.
 * :class:`~ae.core.AppBase` is adding :ref:`application logging` and :ref:`application debugging`.
 
@@ -93,7 +93,7 @@ unit tests
 unit tests need at least V 2.0 of OpenGL and the kivy framework installed.
 
 .. note::
-    unit tests does have 100 % coverage but are currently not passing the gitlab CI tests because we failing in setup
+    unit tests does have 100 % coverage but are currently not passing the gitlab CI tests because were failing in set up
     a proper running window system on the python image that all ae portions are using.
 """
 import os
@@ -124,7 +124,7 @@ from kivy.uix.popup import Popup
 from kivy.uix.slider import Slider                                                          # type: ignore
 import kivy.uix.textinput                                                                   # type: ignore
 # noinspection PyProtectedMember
-from kivy.uix.textinput import TextInput, TextInputCutCopyPaste as _TextInputCutCopyPaste   # type: ignore
+from kivy.uix.textinput import TextInput, TextInputCutCopyPaste as OriTextInputCutCopyPaste
 from kivy.uix.widget import Widget                                                          # type: ignore
 from kivy.utils import escape_markup, get_hex_from_color                                    # type: ignore
 
@@ -138,7 +138,7 @@ from ae.core import DEBUG_LEVELS, DEBUG_LEVEL_ENABLED
 from ae.gui_app import (                                                                    # type: ignore
     APP_STATE_SECTION_NAME, MAX_FONT_SIZE, MIN_FONT_SIZE,
     THEME_LIGHT_BACKGROUND_COLOR, THEME_LIGHT_FONT_COLOR, THEME_DARK_BACKGROUND_COLOR, THEME_DARK_FONT_COLOR,
-    ensure_tap_kwargs_refs, replace_flow_action)
+    ColorOrInk, ensure_tap_kwargs_refs, replace_flow_action)
 from ae.gui_help import HelpAppBase                                                         # type: ignore
 from ae.kivy_glsl import ShaderIdType, ShadersMixin                                         # type: ignore
 from ae.kivy_auto_width import ContainerChildrenAutoWidthBehavior                           # type: ignore
@@ -148,7 +148,7 @@ from ae.kivy_help import (
 from ae.kivy_relief_canvas import relief_colors, ReliefCanvas                               # type: ignore
 
 
-__version__ = '0.2.103'
+__version__ = '0.3.107'
 
 
 MAIN_KV_FILE_NAME = 'main.kv'  #: default file name of the main kv file
@@ -181,7 +181,7 @@ class ImageLabel(ReliefCanvas, Label, ShadersMixin):
 
     .. note::
         hide-able label needs extra handling, because even setting width/height to zero the text can still be visible,
-        especially in dark mode and even with having the text color.alpha==0. to fully hide the texture in all cases,
+        especially in dark mode and even with having the text-color-alpha==0. to fully hide the texture in all cases,
         set either the text to an empty string or the opacity to zero.
     """
 
@@ -230,7 +230,7 @@ class TouchableBehavior:  # pragma: no cover
         super().__init__(**kwargs)      # pylint: disable=no-member
 
         self._app = App.get_running_app()
-        self._state_shader_id: ShaderIdType = dict()
+        self._state_shader_id: ShaderIdType = {}
         self.down_shader = dict(shader_code='=fire_storm', render_shape=Ellipse,
                                 tint_ink=self._app.main_app.flow_path_ink)
         self.normal_shader = dict(shader_code='=plunge_waves', render_shape=Ellipse, add_to='before',
@@ -258,7 +258,7 @@ class TouchableBehavior:  # pragma: no cover
 
     def on_down_shader(self, *_args):
         """ button down state shader changed event handler. """
-        self.on_state(self, self.state)
+        self._update_shader()
 
     def on_long_tap(self, touch: MotionEvent):
         """ long tap/click default handler.
@@ -281,21 +281,15 @@ class TouchableBehavior:  # pragma: no cover
 
     def on_normal_shader(self, *_args):
         """ button normal state shader changed event handler. """
-        self.on_state(self, self.state)
+        self._update_shader()
 
-    def on_state(self, _widget: Any, value: str):
+    def on_state(self, _widget: Any, _value: str):
         """ button pressed state changed event handler, switching between `'normal'` and `'down'` state shader.
 
         :param _widget:         button widget (is self).
-        :param value:           new state value (either 'normal' or 'down').
+        :param _value:          new state value (either 'normal' or 'down').
         """
-        if self._state_shader_id:
-            self.del_shader(self._state_shader_id)
-            self._state_shader_id = dict()
-
-        add_shader_kwargs = self.down_shader if value == 'down' else self.normal_shader
-        if add_shader_kwargs:
-            self._state_shader_id = self.add_shader(**add_shader_kwargs)
+        self._update_shader()
 
     def on_touch_down(self, touch: MotionEvent) -> bool:
         """ check for additional double/triple/alt touch events and add sound, vibration and animation.
@@ -354,6 +348,16 @@ class TouchableBehavior:  # pragma: no cover
         :param touch:           motion/touch event data with the touched widget in `touch.grab_current`.
         """
 
+    def _update_shader(self):
+        """ update shader on changed shader or button state. """
+        if self._state_shader_id:
+            self.del_shader(self._state_shader_id)
+            self._state_shader_id = {}
+
+        add_shader_kwargs = self.down_shader if self.state == 'down' else self.normal_shader
+        if add_shader_kwargs:
+            self._state_shader_id = self.add_shader(**add_shader_kwargs)
+
 
 class FlowButton(HelpBehavior, TouchableBehavior, ButtonBehavior, ImageLabel):  # pragma: no cover
     """ button to change the application flow. """
@@ -367,9 +371,9 @@ class FlowButton(HelpBehavior, TouchableBehavior, ButtonBehavior, ImageLabel):
 
 class FlowDropDown(ContainerChildrenAutoWidthBehavior, DynamicChildrenBehavior, ReliefCanvas,
                    DropDown):  # pragma: no cover
-    """ drop down widget used for user selections from a list of items (represented by the children-widgets). """
+    """ dropdown widget used for user selections from a list of items (represented by the children-widgets). """
     close_kwargs = DictProperty()               #: kwargs passed to all close action flow change event handlers
-    parent_popup_to_close = ObjectProperty()    #: parent popup widget instance to be closed if this drop down closes
+    parent_popup_to_close = ObjectProperty()    #: parent popup widget instance to be closed if this dropdown closes
 
     def dismiss(self, *args):
         """ override DropDown method to prevent dismiss of any dropdown/popup while clicking on activator widget.
@@ -381,7 +385,7 @@ class FlowDropDown(ContainerChildrenAutoWidthBehavior, DynamicChildrenBehavior,
             super().dismiss(*args)
 
     def on_touch_down(self, touch: MotionEvent) -> bool:
-        """ prevent the processing of a touch on the help activator widget by this drop down.
+        """ prevent the processing of a touch on the help activator widget by this dropdown.
 
         :param touch:           motion/touch event data.
         :return:                True if event got processed/used.
@@ -397,7 +401,7 @@ class FlowDropDown(ContainerChildrenAutoWidthBehavior, DynamicChildrenBehavior,
         super()._reposition(*args)
 
 
-class ExtTextInputCutCopyPaste(_TextInputCutCopyPaste):  # pragma: no cover
+class ExtTextInputCutCopyPaste(OriTextInputCutCopyPaste):  # pragma: no cover
     """ overwrite/extend :class:`kivy.uix.textinput.TextInputCutCopyPaste` w/ translatable and autocomplete options. """
     def __init__(self, **kwargs):
         """ create :class:`~kivy.uix.Bubble` instance to display the cut/copy/paste options.
@@ -407,7 +411,7 @@ class ExtTextInputCutCopyPaste(_TextInputCutCopyPaste):  # pragma: no cover
         endless recursion because else the other super(cls, instance) call (in python2 style within
         :meth:`TextInputCutCopyPaste.__init__`) results in the same instance (instead of the overwritten instance).
         """
-        kivy.uix.textinput.TextInputCutCopyPaste = _TextInputCutCopyPaste
+        kivy.uix.textinput.TextInputCutCopyPaste = OriTextInputCutCopyPaste
         self.fw_app = App.get_running_app()
         super().__init__(**kwargs)
 
@@ -459,28 +463,30 @@ class FlowInput(HelpBehavior, TextInput, ShadersMixin):  # pragma: no cover
    to implement a dark background for the dark theme we would need also to change the images in the properties:
     background_active, background_disabled_normal and self.background_normal.
 
-    also the images/colors of the bubble that is showing e.g. on long press of the TextInput widget (cut/copy/paste/...)
+    the images/colors of the bubble that is showing e.g. on long press of the TextInput widget (cut/copy/paste/...)
     kept unchanged - only the font_size get adapted and the bubble button texts get translated. for that the class
     :class:`ExtTextInputCutCopyPaste` provided by this portion inherits from the original bubble class
-    :class:`~kivy.uix.textinput.TextInputCutCopyPaste`. additionally the original bubble class gets monkey patched
-    shortly/temporarily in the moment of the instantiation to translate the bubble menu options, change the font
-    sizes and add additional menu options to memorize/forget auto-completion texts.
+    :class:`~kivy.uix.textinput.TextInputCutCopyPaste`.
+
+    the original bubble class is getting monkey patched shortly/temporarily in the moment of the instantiation to
+    translate the bubble menu options, change the font sizes and add additional menu options to memorize/forget
+    auto-completion texts.
     """
     focus_flow_id = StringProperty()        #: flow id that will be set when this widget get focus
     unfocus_flow_id = StringProperty()      #: flow id that will be set when this widget lost focus
 
     auto_complete_texts: List[str] = ListProperty()  #: list of autocompletion texts
-    auto_complete_selector_index_ink: Tuple[float, float, float, float] = ListProperty((0.69, 0.69, 0.69, 1))
+    auto_complete_selector_index_ink: ColorOrInk = ListProperty([0.69, 0.69, 0.69, 1])
     """ color and alpha used to highlight the currently selected text of all matching autocompletion texts """
 
     _ac_dropdown: Any = None                #: singleton FlowDropDown instance for all TextInput instances
-    _matching_ac_texts: List[str] = list()  #: one list instance for all TextInput instances is enough
-    _matching_ac_index: int = 0             #: index of selected text in the drop down matching texts list
+    _matching_ac_texts: List[str] = []      #: one list instance for all TextInput instances is enough
+    _matching_ac_index: int = 0             #: index of selected text in the dropdown matching texts list
 
     def __init__(self, **kwargs):
         # changed to kivy properties so no need to pop them from kwargs:
-        # self.auto_complete_texts = kwargs.pop('auto_complete_texts', list())
-        # self.auto_complete_selector_index_ink = kwargs.pop('auto_complete_selector_index_ink', (0.69, 0.69, 0.69, 1))
+        # self.auto_complete_texts = kwargs.pop('auto_complete_texts', [])
+        # self.auto_complete_selector_index_ink = kwargs.pop('auto_complete_selector_index_ink', [0.69, 0.69, 0.69, 1.])
 
         super().__init__(**kwargs)
 
@@ -510,7 +516,7 @@ class FlowInput(HelpBehavior, TextInput, ShadersMixin):  # pragma: no cover
             ac_text = self._matching_ac_texts[self._matching_ac_index]
         if ac_text in self.auto_complete_texts:
             self.auto_complete_texts.remove(ac_text)
-            self.on_text(self, self.text)  # type: ignore  # redraw autocompletion dropdown
+            self.on_text(self, self.text)   # redraw autocompletion dropdown
 
     def delete_text_from_ac(self, *_args):
         """ check if current text is in autocompletion list and if yes then remove it.
@@ -583,12 +589,12 @@ class FlowInput(HelpBehavior, TextInput, ShadersMixin):  # pragma: no cover
         if text:
             matching = [txt for txt in self.auto_complete_texts if txt[:-1].startswith(text)]
         else:
-            matching = list()
+            matching = []
         self._matching_ac_texts[:] = matching
         self._matching_ac_index = 0
 
         if matching:
-            cdm = list()
+            cdm = []
             for txt in matching:
                 cdm.append(dict(cls='FlowButton', kwargs=dict(text=txt, on_release=self._select_ac_text)))
             self._ac_dropdown.child_data_maps[:] = cdm
@@ -609,13 +615,13 @@ class FlowInput(HelpBehavior, TextInput, ShadersMixin):  # pragma: no cover
     def _show_cut_copy_paste(self, *args, **kwargs):    # pylint: disable=signature-differs
         kivy.uix.textinput.TextInputCutCopyPaste = ExtTextInputCutCopyPaste  # reset in ExtTextInputCutCopyPaste.__init_
         super()._show_cut_copy_paste(*args, **kwargs)
-        kivy.uix.textinput.TextInputCutCopyPaste = _TextInputCutCopyPaste  # reset here too if already instantiated
+        kivy.uix.textinput.TextInputCutCopyPaste = OriTextInputCutCopyPaste  # reset here too if already instantiated
 
 
 class FlowPopup(ModalBehavior, DynamicChildrenBehavior, ReliefCanvas, BoxLayout):                   # pragma: no cover
     """ popup for dynamic and auto-content-sizing dialogs and other top-most or modal windows.
 
-    the scrollable :attr:`container` (a :class:`~kivy.uix.scrollview.ScrollView` instance) can only have one children,
+    the scrollable :attr:`container` (a :class:`~kivy.uix.scrollview.ScrollView` instance) can only have one child,
     the content. use a layout as content to display multiple widgets. set :attr:`content_optimal_width` and/or
     :attr:`content_optimal_height` to make the popup size as small as possible, using e.g. `minimum_width` respectively
     `minimum_height` if the content is a layout that is providing and updating this property, or
@@ -643,7 +649,7 @@ class FlowPopup(ModalBehavior, DynamicChildrenBehavior, ReliefCanvas, BoxLayout)
         `on_pre_dismiss`:
             fired before the FlowPopup is closed.
         `on_dismiss`:
-            fired when the FlowPopup is closed. if the callback returns True, the dismiss will be canceled.
+            fired when the FlowPopup is closed. if the callback returns True, the popup will stay opened.
 
     """
 
@@ -746,7 +752,7 @@ class FlowPopup(ModalBehavior, DynamicChildrenBehavior, ReliefCanvas, BoxLayout)
 
         clr_ink = Window.clearcolor
         self.background_color = clr_ink
-        self.overlay_color = tuple(clr_ink[:3]) + (.6, )
+        self.overlay_color = clr_ink[:3] + [0.6]
         self.relief_square_outer_colors = relief_colors(app.font_color)
         self.relief_square_outer_lines = sp(9)
         self.separator_color = app.font_color
@@ -872,7 +878,7 @@ class FrameworkApp(App):
     landscape = BooleanProperty()                       #: True if app win width is bigger than the app win height
     max_font_size = NumericProperty(MAX_FONT_SIZE)      #: maximum font size in pixels bound to window size
     min_font_size = NumericProperty(MIN_FONT_SIZE)      #: minimum - " -
-    mixed_back_ink = ListProperty((.69, .69, .69, 1.))  #: background color mixed from available back inks
+    mixed_back_ink = ListProperty([.69, .69, .69, 1.])  #: background color mixed from available back inks
     tour_layout = ObjectProperty(allownone=True)        #: overlay layout widget if tour is active else None
 
     def __init__(self, main_app: 'KivyMainApp', **kwargs):
@@ -1078,20 +1084,20 @@ class _GetTextBinder(Observable):
         :param language:        language code to translate the passed text to (def=current default language).
         :param loc_vars:        local variables used in the conversion of the f-string expression to a string.
                                 the `count` item of this dict will be overwritten by the value of the
-                                :paramref:`~_GetTextBinder.__call__.count` parameter (if this argument got passed).
+                                :paramref:`~_GetTextBinder.__call__.count` parameter (if this argument got specified).
         :param kwargs:          extra kwargs (e.g. :paramref:`~ae.i18n.get_f_string.glo_vars` or
                                 :paramref:`~ae.i18n.get_f_string.key_suffix` - see :func:`~ae.i18n.get_f_string`).
         :return:                translated text.
         """
         if count is not None:
             if loc_vars is None:
-                loc_vars = dict()
+                loc_vars = {}
             loc_vars['count'] = count
         return get_f_string(text, language=language, loc_vars=loc_vars, **kwargs)
 
 
 get_txt = _GetTextBinder()              #: instantiate global i18n translation callable and language switcher helper
-get_txt.__qualname__ = 'GetTextBinder'  # hide sphinx build warning (build crashes if get_txt get documented)
+get_txt.__qualname__ = 'GetTextBinder'  # hide sphinx build warning (build crashes if the get_txt var get documented)
 global_idmap['_'] = get_txt             # bind as function/callable with the name `_` to be used in kv files
 
 
@@ -1141,12 +1147,12 @@ class KivyMainApp(HelpAppBase):
                 app_states_data["framework app states"] = self.framework_app.app_states
                 app_states_data['kbd_input_mode'] = self.kbd_input_mode
 
-                help_data = dict()
-                help_data["global_variables"] = self.global_variables()
-                help_data['_last_focus_flow_id'] = self._last_focus_flow_id
-                help_data['_next_help_id'] = self._next_help_id
-                help_data['displayed_help_id'] = self.displayed_help_id
-                app_env_info['help data'] = help_data
+                app_env_info['help data'] = {
+                    'displayed_help_id': self.displayed_help_id,
+                    'global_variables': self.global_variables(),
+                    '_last_focus_flow_id': self._last_focus_flow_id,
+                    '_next_help_id': self._next_help_id,
+                }
 
                 app_env_info['app data']['documents_root_path'] = self.documents_root_path
             app_env_info['app states data'] = app_states_data
@@ -1215,7 +1221,7 @@ class KivyMainApp(HelpAppBase):
         tour_layout = self.tour_layout
         activate = help_layout is None and tour_layout is None
         help_id = ''
-        help_vars = dict()
+        help_vars = {}
         if activate:
             target, help_id = self.help_target_and_id(help_vars)
             help_layout = Tooltip(targeted_widget=target)
@@ -1240,12 +1246,12 @@ class KivyMainApp(HelpAppBase):
             activator.ani_start()
 
     def load_sounds(self):
-        """ override to pre-load audio sounds from app folder snd into sound file cache. """
+        """ override to preload audio sounds from app folder snd into sound file cache. """
         super().load_sounds()  # load from sound file paths all files into :class:`~ae.files.RegisteredFile` instances
         self.sound_files.reclassify(object_loader=lambda f: SoundLoader.load(f.path))  # :class:`~ae.files.CachedFile`
 
     def on_app_build(self):
-        """ kivy App build event handler called at the begin of :meth:`kivy.app.App.build`. """
+        """ kivy App build event handler called at the beginning of :meth:`kivy.app.App.build`. """
         super().on_app_build()
         self.vpo("KivyMainApp.on_app_build - reload image resources from kv file late imports, e.g. ae.kivy_user_prefs")
         self.load_images()
@@ -1331,7 +1337,7 @@ class KivyMainApp(HelpAppBase):
         if not self.debug:
             self._debug_enable_clicks += 1
             if self._debug_enable_clicks >= 3:
-                self.on_debug_level_change(DEBUG_LEVELS[DEBUG_LEVEL_ENABLED], dict())  # also enable for all sub-apps
+                self.on_debug_level_change(DEBUG_LEVELS[DEBUG_LEVEL_ENABLED], {})  # also enable for all sub-apps
                 self._debug_enable_clicks = 0
             elif self._debug_enable_clicks == 1:
                 Clock.schedule_once(_timeout_reset, 6.0)
@@ -1362,7 +1368,7 @@ class KivyMainApp(HelpAppBase):
         """ play vibrate pattern. """
         self.vpo(f"KivyMainApp.play_vibrate {pattern}")
         if self.framework_app.app_states.get('vibration_volume', 1.):  # no volume available, at least disable if 0.0
-            try:  # added because is crashing with current plyer version (master should work)
+            try:  # added because it's crashing with current plyer version (master should work)
                 vibrator.pattern(pattern)
             # except jnius.jnius.JavaException as ex:
             #    self.po(f"KivyMainApp.play_vibrate JavaException {ex}, update plyer to git/master")
@@ -1388,8 +1394,8 @@ class KivyMainApp(HelpAppBase):
         :param popup_class:     class of the Popup or DropDown widget.
         :param popup_kwargs:    args to be set as attributes of the popup class instance plus an optional
                                 `parent` kwarg that will be passed as the popup parent widget arg
-                                to the popup.open method; if parent does not get passed then the root widget/layout
-                                of self.framework_app will passed into the popup.open method as the widget argument.
+                                to the popup.open() method; if parent gets not specified then the root widget/layout
+                                of self.framework_app will be passed into popup.open() as the widget argument.
         :return:                created and displayed/opened popup class instance.
         """
         self.dpo(f"KivyMainApp.open_popup {popup_class} {popup_kwargs}")
